@@ -1467,7 +1467,18 @@ static void fuse_ll_process(void *data, const char *buf, size_t len,
 
 	req = (struct fuse_req *) calloc(1, sizeof(struct fuse_req));
 	if (req == NULL) {
+		struct fuse_out_header out = {
+			.unique = in->unique,
+			.error = -ENOMEM,
+			.len = sizeof(struct fuse_out_header),
+		};
+		struct iovec iov = {
+			.iov_base = &out,
+			.iov_len = sizeof(struct fuse_out_header),
+		};
+
 		fprintf(stderr, "fuse: failed to allocate request\n");
+		fuse_chan_send(ch, &iov, 1);
 		return;
 	}
 
@@ -1728,7 +1739,7 @@ out_free:
 /*
  * This is currently not implemented on other than Linux...
  */
-int fuse_req_getgroups(fuse_req_t req, int size, gid_t list[]);
+int fuse_req_getgroups(fuse_req_t req, int size, gid_t list[])
 {
 	return -ENOSYS;
 }
